;**** **** **** **** ****
;
;Die Benutzung der Software ist mit folgenden Bedingungen verbunden:
;
;1. Da ich alles kostenlos zur Verfgung stelle, gebe ich keinerlei Garantie
;   und bernehme auch keinerlei Haftung fr die Folgen der Benutzung.
;
;2. Die Software ist ausschlielich zur privaten Nutzung bestimmt. Ich
;   habe nicht geprft, ob bei gewerblicher Nutzung irgendwelche Patentrechte
;   verletzt werden oder sonstige rechtliche Einschrnkungen vorliegen.
;
;3. Jeder darf nderungen vornehmen, z.B. um die Funktion seinen Bedrfnissen
;   anzupassen oder zu erweitern. Ich wrde mich freuen, wenn ich weiterhin als
;   Co-Autor in den Unterlagen erscheine und mir ein Link zur entprechenden Seite
;   (falls vorhanden) mitgeteilt wird.
;
;4. Auch nach den nderungen sollen die Software weiterhin frei sein, d.h. kostenlos bleiben.
;
;!! Wer mit den Nutzungbedingungen nicht einverstanden ist, darf die Software nicht nutzen !!
;
; April 2007
; autor: Bernhard Konze
; email: bernhard.konze(at)versanet.de
;
;**** **** **** **** ****
; Device
;
;**** **** **** **** ****
.include "tn13def.inc"
;

;AVR  -> 10pol Wannenstecker (AVRISP-STK200)
;pin1 -> pin5-(Reset)
;pin4 -> pin4,6,8,10-(GND)
;pin5 -> pin1-(MOSI)
;pin6 -> pin9-(MISO)
;pin7 -> pin7-(SCK)
;pin8 -> pin2-(Vcc)

;AVR  -> control:input-output
;pin2 -> PB4 = pwm-in1
;pin3 -> PB3 = pwm-in2.
;pin5 -> PB0 = ppm-out2
;pin6 -> PB1 = ppm-out1
;pin7 -> PB2 = LED

.equ INIT_PB	= 0x00 	; all inputs low = no internal pull-up
.equ DIR_PB	= 0x07	; PPM = output LED = on

; PWM input
.equ PWM_IN1	= 4
.equ PWM_IN2	= 3
; PPM output
.equ PPM_OUT1	= 1
.equ PPM_OUT2	= 0
; LED output
.equ LED_OUT	= 2

.equ PPM_MS_REL	= 150
.equ PPM_PERIOD	= 20	; means 19-20 ms

.equ CCM_AB_EN	= (1<<OCIE0B)+(1<<OCIE0A)
.equ CCM_B_EN	= (1<<OCIE0B)
.equ CCM_A_RES	= (1<<OCF0A)



;**** **** **** **** ****
; Register Definitions
.def	i_sreg		= r0		; status register save in interrupts

.def	on1cnt_h	= r1
.def	on1cnt_l	= r2
.def	on2cnt_h	= r3
.def	on2cnt_l	= r4
.def	loop_h		= r5
.def	loop_l		= r6

.def	ppm1		= r7
.def	ppm2		= r8

.def	temp1		= r16		; main temporary
.def	temp2		= r17		; main temporary
.def	i_temp1		= r18		; interrupt temporary
.def	i_temp2		= r19		; interrupt temporary

.def	ppm_ms		= r20

.def	led_cnt		= r21

.def	flags0	= r25	; state flags
	.equ	PPM_SEND	= 0	; if set, PPM1+2 were just send out
	.equ	PPM2_INIT	= 1	; if set, PPM2 is prepared


; ***** INTERRUPT VECTORS ************************************************
;.equ	INT0addr	= 0x0001	; External Interrupt 0
;.equ	PCI0addr	= 0x0002	; External Interrupt Request 0
;.equ	OVF0addr	= 0x0003	; Timer/Counter0 Overflow
;.equ	ERDYaddr	= 0x0004	; EEPROM Ready
;.equ	ACIaddr		= 0x0005	; Analog Comparator
;.equ	OC0Aaddr	= 0x0006	; Timer/Counter Compare Match A
;.equ	OC0Baddr	= 0x0007	; Timer/Counter Compare Match B
;.equ	WDTaddr		= 0x0008	; Watchdog Time-out
;.equ	ADCCaddr	= 0x0009	; ADC Conversion Complete


;-----bko-----------------------------------------------------------------

;**** **** **** **** ****
.cseg
.org 0
;**** **** **** **** ****

;-----bko-----------------------------------------------------------------
; reset and interrupt jump table
		rjmp	reset
		nop	; ext0_int
		nop	; pci0_int
		nop	; ovf0_int
		nop	; erdy
		nop	; aci
		rjmp	ccma_int
		rjmp	ccmb_int
		nop	; wdc
		nop	; adc


version:	.db	"koax13_bko01"
		.db	"#20ms#"


;-----bko-----------------------------------------------------------------
; init after reset

reset:	; OSCCAL is automatically set during reset

	; portB: has to carry all weight
		ldi	temp1, INIT_PB
		out	PORTB, temp1
		ldi	temp1, DIR_PB
		out	DDRB, temp1

; we use the internal calibrated clock with 9.6MHz

	; timer0: free running clock
	;  with 9.6MHz/64 =^= 6.67s/count =^= 150counts/ms
		ldi	temp1, 0x00	; no special mode
		out	TCCR0A, temp1
		ldi	temp1, 0x03	; clk/64
		out	TCCR0B, temp1

	; init vars
		ldi	ppm_ms, PPM_PERIOD
		ldi	temp1, 0x00
		mov	ppm1, temp1
		mov	ppm2, temp1

	; init registers and interrupts
		ldi	temp1, CCM_B_EN
		out	TIFR0, temp1
		out	TIMSK0, temp1

		sei				; enable all interrupts

		ldi	ppm_ms, 200
wait1:	cpi	ppm_ms, 10	; match the appropriate time
		brne wait1
		ldi	ppm_ms, 200
wait2:	cpi	ppm_ms, 10	; match the appropriate time
		brne wait2
		ldi	ppm_ms, 200
wait3:	cpi	ppm_ms, 10	; match the appropriate time
		brne wait3

		ldi	ppm_ms, PPM_PERIOD

		ldi	led_cnt, 25


		rjmp	cnv_PWM2PPM

;-----bko-----------------------------------------------------------------
; counter compare match B interrupt
; PPM output timing
ccmb_int:	in	i_sreg, SREG
	; evaluate state
		dec	ppm_ms
		cpi	ppm_ms, 2	; ppm high ?
		breq	ccmb_ppm_1
		cpi	ppm_ms, 1	; ppm length ?
		breq	ccmb_ppm_2
		cpi	ppm_ms, 0	; reset ppm1 + reload state ?
		brne	ccmb_i90

		cbi	PORTB, PPM_OUT1
		ldi	ppm_ms, PPM_PERIOD
		sbr	flags0, (1<<PPM_SEND)
		rjmp	ccmb_i90

ccmb_ppm_1:	sbi	PORTB, PPM_OUT1
		sbi	PORTB, PPM_OUT2
		in	i_temp1, OCR0B
		subi	i_temp1, -(PPM_MS_REL-20)
		out	OCR0A, i_temp1	; compare 0A comes first
		subi	i_temp1, -10
		out	OCR0B, i_temp1
		ldi	i_temp1, CCM_A_RES ; enable OCR0A
		out	TIFR0, i_temp1
		ldi	i_temp1, CCM_AB_EN
		out	TIMSK0, i_temp1
		sbr	flags0, (1<<PPM2_INIT)
		rjmp	ccmb_i99

ccmb_ppm_2:	in	i_temp1, OCR0B
		add	i_temp1, ppm1	; ppm1 must have been added by 10
		out	OCR0B, i_temp1
		rjmp	ccmb_i99

	; set to next millisecond
ccmb_i90:	in	i_temp1, OCR0B
		subi	i_temp1, -PPM_MS_REL
		out	OCR0B, i_temp1

ccmb_i99:	out	SREG, i_sreg
		reti
;-----bko-----------------------------------------------------------------
; counter compare match A interrupt
; PWM input timing
ccma_int:	in	i_sreg, SREG

		sbrc	flags0, PPM2_INIT
		rjmp	ccma_ppm_2

		cbi	PORTB, PPM_OUT2	; all done
		ldi	i_temp1, CCM_B_EN ; disable CCM_A
		out	TIMSK0, i_temp1
		rjmp	ccma_i99

ccma_ppm_2:	in	i_temp1, OCR0A
		add	i_temp1, ppm2	; ppm2 must have been added by 20
		out	OCR0A, i_temp1
		cbr	flags0, (1<<PPM2_INIT)
		rjmp	ccma_i99

ccma_i99:	out	SREG, i_sreg
		reti

;-----bko-----------------------------------------------------------------
cnv_PWM2PPM_1:	cpi	ppm_ms, 6	; match the appropriate time
		brne	cnv_PWM2PPM

cnv_PWM2PPM:	ldi	temp1, low  (-150*16)
		ldi	temp2, high (-150*16)
		mov	loop_l, temp1
		mov	loop_h, temp2
		ldi	temp1, 0
		mov	on1cnt_l, temp1
		mov	on1cnt_h, temp1
		mov	on2cnt_l, temp1
		mov	on2cnt_h, temp1
		clr	temp2

count_on:	clr	temp1			; PWM 1
		sbis	PINB, PWM_IN1
		ldi	temp1, 0
		sbic	PINB, PWM_IN1
		ldi	temp1, 1
		add	on1cnt_l, temp1
		adc	on1cnt_h, temp2

		clr	temp1			; PWM 2
		sbis	PINB, PWM_IN2
		ldi	temp1, 0
		sbic	PINB, PWM_IN2
		ldi	temp1, 1
		add	on2cnt_l, temp1
		adc	on2cnt_h, temp2

		inc	loop_l
		brne	count_on
		inc	loop_h
		brne	count_on

		lsr	on1cnt_h
		ror	on1cnt_l
		lsr	on1cnt_h
		ror	on1cnt_l
		lsr	on1cnt_h
		ror	on1cnt_l
		lsr	on1cnt_h
		ror	on1cnt_l

		mov	temp1, on1cnt_l
		subi	temp1, -10
		mov	ppm1, temp1

		lsr	on2cnt_h
		ror	on2cnt_l
		lsr	on2cnt_h
		ror	on2cnt_l
		lsr	on2cnt_h
		ror	on2cnt_l
		lsr	on2cnt_h
		ror	on2cnt_l

		mov	temp1, on2cnt_l
		subi	temp1, -20
		mov	ppm2, temp1

		dec	led_cnt
		brne	cnv_PWM2PPM
		ldi	led_cnt, 25
		sbic	PORTB, LED_OUT
		rjmp	cnv_LEDon
		sbi	PORTB, LED_OUT
		rjmp	cnv_PWM2PPM
cnv_LEDon:	cbi	PORTB, LED_OUT
		rjmp	cnv_PWM2PPM
;-----bko-----------------------------------------------------------------
.exit
